import unittest
import time
import thespian.test.helpers
from thespian.actors import *
from thespian.test import ActorSystemTestCase


class PoisonCounter(Actor):
    def __init__(self, *args, **kw):
        super(PoisonCounter, self).__init__(*args, **kw)
        self.num_poisoned = 0
        self.failed = None
    def receiveMessage(self, msg, sender):
        if "Count?" == msg:
            if self.failed:
                self.send(sender, self.failed)
            else:
                self.send(sender, self.num_poisoned)
        elif "Poisoned" == msg:
            self.num_poisoned += 1
        elif type(msg) == type((1,2)) and "PoisonMostFoul" == msg[0]:
            self.failed = msg[1]


class TestActor(Actor):
    def receiveMessage(self, msg, sender):
        if isinstance(msg, PoisonMessage):
            if sender == self.myAddress:
                self.send(self.recorder, ("PoisonMostFoul", "Got PoisonMessage from self!"))
            else:
                self.send(self.recorder, "Poisoned")
        elif isinstance(msg, ActorAddress):
            self.recorder = msg
        elif isinstance(msg, ChildActorExited):
            if not msg.isPermanent():
                msg.replaceActor(TestActor)
        elif isinstance(msg, int):
            if msg == 3:
                raise ValueError('Yucky number 3')
            next = self.createActor(TestActor)
            self.send(next, self.recorder)
            self.send(next, msg+1)
        else:
            pass


class NoBadNewsActor(TestActor):
    def receiveMessage(self, msg, sender):
        if isinstance(msg, ChildActorExited):
            raise IndexError('Do not give me bad news')
        super(NoBadNewsActor, self).receiveMessage(msg, sender)


class TestASimpleSystem(ActorSystemTestCase):

    def waitForCount(self, counter, newCount):
        aS = ActorSystem()
        for X in range(50):
            cnt = aS.ask(counter, "Count?")
            if newCount == cnt:
                break
            time.sleep(0.01)
        self.assertEqual(aS.ask(counter, "Count?"), newCount)

    def testBadNews(self):
        aS = ActorSystem()
        counter = aS.createActor(PoisonCounter)
        t1 = aS.createActor(TestActor)
        aS.tell(t1, counter)
        self.assertEqual(aS.ask(counter, "Count?"), 0)
        aS.tell(t1, 1)
        self.waitForCount(counter, 1)
        aS.tell(t1, ActorExitRequest())


    def testLevel2(self):
        aS = ActorSystem()
        counter = aS.createActor(PoisonCounter)
        t1 = aS.createActor(TestActor)
        aS.tell(t1, counter)
        self.assertEqual(aS.ask(counter, "Count?"), 0)
        aS.tell(t1, 2)
        self.waitForCount(counter, 1)
        aS.tell(t1, 2)
        aS.tell(t1, 2)
        self.waitForCount(counter, 3)


    def testLevel1(self):
        aS = ActorSystem()
        counter = aS.createActor(PoisonCounter)
        t1 = aS.createActor(TestActor)
        aS.tell(t1, counter)
        self.assertEqual(aS.ask(counter, "Count?"), 0)
        aS.tell(t1, 1)
        self.waitForCount(counter, 1)
        aS.tell(t1, 1)
        aS.tell(t1, 1)
        self.waitForCount(counter, 3)

    def testLevel3(self):
        "No-one to send PoisonMessage notification to for top level actor (or to restart it)"
        aS = ActorSystem()
        counter = aS.createActor(PoisonCounter)
        t1 = aS.createActor(TestActor)
        aS.tell(t1, counter)
        self.assertEqual(aS.ask(counter, "Count?"), 0)
        aS.tell(t1, 3)
        # Died, and dead-lettered
        time.sleep(0.01)
        self.assertEqual(aS.ask(counter, "Count?"), 0)
        aS.tell(t1, 3)
        aS.tell(t1, 3)
        time.sleep(0.01)
        self.assertEqual(aS.ask(counter, "Count?"), 0)


#until multiprocess system works on Windows
import os
if os.name!='nt':
    class TestMultiProcessSystem(TestASimpleSystem):
        def setUp(self):
            self.setSystemBase('multiprocessSystemBase')
